package com.example.demo.users.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.core.configuration.Constants;
import com.example.demo.games.model.GameEntity;
import com.example.demo.games.service.GameService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

@RestController
//RequestMapping - доступ ко всем методам контроллера только по адресу в скобках
@RequestMapping(Constants.API_URL + "/user")
public class UserController {
    //бизнес-логика
    private final UserService userService;

    private final GameService gameService;

    private final ModelMapper modelMapper;

    public UserController(UserService userService, ModelMapper modelMapper, GameService gameService) {
        this.userService = userService;
        this.gameService = gameService;
        this.modelMapper = modelMapper;
    }

    private UserDto toDto(UserEntity entity) {
        // return modelMapper.map(entity, UserDto.class);
        final var dto = new UserDto();
        dto.setId(entity.getId());
        dto.setName(entity.getName());
        dto.setEmail(entity.getEmail());
        dto.setPassword(entity.getPassword());
        dto.setGames(entity.getGames().stream().map(GameEntity::getId).toList());
        return dto;
    }

    private UserEntity toEntity(UserDto dto) {
        return modelMapper.map(dto, UserEntity.class);
    }

    //GetMapping - обработка HTTP GET-запросов; более короткий аналог RequestMapping(value = "/", method = RequestMethod.GET)
    @GetMapping
    public List<UserDto> getAll() {
        return userService.getAll().stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public UserDto get(@PathVariable(name = "id") Long id) {
        return toDto(userService.get(id));
    }

    //PostMapping - добавление
    @PostMapping
    public UserDto create(@Valid @RequestBody UserDto dto) {
        return toDto(userService.create(toEntity(dto)));
    }

    //PostMapping - добавление
    @PostMapping("/{id}/game/{gameId}")
    public UserDto addGame(@PathVariable(name="id") Long id, @PathVariable(name="gameId") Long gameId) {
        return toDto(userService.addGame(id, gameService.get(gameId)));
    }

    //PutMapping - обновление
    @PutMapping("/{id}")
    public UserDto update(@PathVariable(name = "id") Long id, @Valid @RequestBody UserDto dto) {
        return toDto(userService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public UserDto delete(@PathVariable(name = "id") Long id) {
        return toDto(userService.delete(id));
    }

    @DeleteMapping("/{id}/game/{gameId}")
    public UserDto deleteGame(@PathVariable(name="id") Long id, @PathVariable(name="gameId") Long gameId) {
        return toDto(userService.deleteGame(id, gameService.get(gameId)));
    }
}
