package com.example.demo;


import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.DependsOn;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.games.model.GameEntity;
import com.example.demo.games.service.GameService;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;

//DependsOn("TypeServiceTests") - запуск теста после TypeServiceTests
//В некоторых местах 4L, т.к. последний жанр после удаления и обратного добавления будет иметь id не 3, а 4
@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
// @DependsOn("TypeServiceTests")
public class GameServiceTests {
    //Автоматическое внедрение зависимостей в сервис
	@Autowired
	private GameService gameService;

    @Autowired
	private TypeService typeService;

    //Проверка того, что get выдаст кастомное исключение, если в него передать несуществующий id 
	@Test
	void getTest() {
		Assertions.assertThrows(NotFoundException.class, () -> gameService.get(0L));
	}

    //Проверка добавления в хранилище
    @Test
	@Order(1)
	void createTest() {
		typeService.deleteAll();
		gameService.deleteAll();

		typeService.create(new TypeEntity(null, "FMV"));
		typeService.create(new TypeEntity(null, "Экшен"));
		typeService.create(new TypeEntity(null, "Стратегия"));

		gameService.create(new GameEntity(null, typeService.get(1L), "Пробная игра 1", 100, "Пробное описание 1"));

		gameService.create(new GameEntity(null, typeService.get(2L), "Пробная игра 2", 450, "Пробное описание 2"));

        final GameEntity last = gameService.create(new GameEntity(null, typeService.get(3L), "Пробная игра 3", 330, "Пробное описание 3"));

		//проверка, что кол-во игр в хранилище равно кол-ву добавленных (проверка, что все добавились)
		Assertions.assertEquals(3, gameService.getAll(0L).size());
		//проверка, что в хранилище последняя игра та, которую добавили последней (проверка, что всё правильно добавилось)
		Assertions.assertEquals(last, gameService.get(3L));
	}


    //Проверка правильности обновления на примере последней игры (замена значений всех полей)
    @Test
	@Order(2)
	void updateTest() {
		//на какой жанр будем менять (со Стратегии на FMV)
        final TypeEntity testEntity = typeService.get(1L);

        //на какое название будем менять название игры
		final String testTitle = "TEST";

        //на какую цену будем менять цену игры
        final Integer testPrice = 1000;

        //на какое описание будем менять описание игры
        final String testDescription = "TEST DESCRIPTION";


		//получение последней в списке игры
		final GameEntity entity = gameService.get(3L);

        //запоминание старого жанра игры
        final TypeEntity oldTypeEntity = entity.getType();

		//запоминание её названия до обновления
		final String oldTitle = entity.getTitle();

        //запоминание её цены до обновления
        final Integer oldPrice = entity.getPrice();

        //запоминание её описания до обновления
        final String oldDescription = entity.getDescription();

		//запоминание новых данных жанра
		final GameEntity newEntity = gameService.update(3L, new GameEntity(1L, testEntity, testTitle, testPrice, testDescription));

		//Проверка, что кол-во игр до сих пор 3
		Assertions.assertEquals(3, gameService.getAll(0L).size());


		Assertions.assertEquals(newEntity, gameService.get(3L));

        //Проверка, что жанр поменялся
        Assertions.assertEquals(testEntity, newEntity.getType());
        Assertions.assertNotEquals(oldTypeEntity, newEntity.getType());

		//Проверка, что название поменялось
		Assertions.assertEquals(testTitle, newEntity.getTitle());
        Assertions.assertNotEquals(oldTitle, newEntity.getTitle());

        //Проверка, что цена поменялась
		Assertions.assertEquals(testPrice, newEntity.getPrice());
        Assertions.assertNotEquals(oldPrice, newEntity.getPrice());

        //Проверка, что описание поменялось
        Assertions.assertEquals(testDescription, newEntity.getDescription());
        Assertions.assertNotEquals(oldDescription, newEntity.getDescription());
	}

    //Проверка правильности удаления на примере последней игры
	@Test
	@Order(3)
	void deleteTest() {
		gameService.delete(3L);
		//Проверка, что после удаления в хранилище осталось 2 игры
		Assertions.assertEquals(2, gameService.getAll(0L).size());

		final GameEntity last = gameService.get(2L);
		Assertions.assertEquals(2L, last.getId());

		final GameEntity newEntity = gameService.create(new GameEntity(null, typeService.get(3L), "Пробная игра 3", 330, "Пробное описание 3"));
		//Проверка, что в хранилище теперь снова 3 игры
		Assertions.assertEquals(3, typeService.getAll().size());
		//Проверка, что id у добавленной игры стал 4
		Assertions.assertEquals(4L, newEntity.getId());
	}
}
